using System;
using System.Collections.Generic;
using System.Text;

// Allow dll import
using System.Runtime.InteropServices;

namespace LogicIO.RTCU.DllImport
{
    /// <summary>
    /// Different return codes from the gateway functions.
    /// </summary>
    public enum GWRC : int
    {
        OK = 0,           /* operation successful */
        ERROR = 1,        /* unspecified error */
        NOT_CON = 2,      /* not connected */
        TIMEOUT = 3,      /* a timeout occurred */
        INV_LEN = 5,      /* invalid length */
        IS_OPEN = 7,      /* is already open */
        NOT_OPEN = 8,     /* is not open */
        INV_PARM = 9,     /* invalid parameters */
        DST_UNREACH = 10, /* destination node is unreachable */
    }

    /// <summary>
    /// Allow simple communication to RTCU units over the RTCU Gateway
    /// 
    /// Encapsulates sending/receiving of VSMS messages to RTCU Units through the Logic IO RTCU Gateway 
    /// <remarks>It can only be initialized once, to reinitialize the VSMSGW.DLL must be reloaded</remarks>
    /// </summary>
    public static class VSMSGW
    {
        #region "Delegates for callback functions"
        /// <summary>
        /// Definition of callback function for receiving Text SMS messages
        /// </summary>
        /// <param name="HisNodeID">The nodeid of the sender (the serialnumber of the RTCU that sent the message)</param>
        /// <param name="str">The text string sent by the RTCU.Max size is 160 characters + a null-terminator</param>
        /// <param name="arg">A user supplied 32 bit variable that was set when vsmsgwInit() was called</param>
        /// <returns>This is the return code that should be returned to the RTCU Unit, 0 if OK, else error</returns>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate UInt16 cbfuncText(
            Int32 HisNodeID,
            [MarshalAs(UnmanagedType.LPStr)] String str,
            IntPtr arg
        );

        /// <summary>
        /// Definition of callback function for receiving PDU SMS messages
        /// </summary>
        /// <param name="HisNodeID">The nodeid of the sender (the serialnumber of the RTCU that sent the message)</param>
        /// <param name="data">The data sent by the RTCU.Max size is 140 bytes.</param>
        /// <param name="length">The length of data sent by the RTCU</param>
        /// <param name="arg">A user supplied 32 bit variable that was set when vsmsgwInit() was called</param>
        /// <returns>This is the return code that should be returned to the RTCU Unit, 0 if OK, else error</returns>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate UInt16 cbfuncPDU(
            int HisNodeID,
            [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] byte[] data,
            int length,
            IntPtr arg
        );

        /// <summary>
        /// Definition of callback function for receiving packets
        /// </summary>
        /// <param name="HisNodeID">The nodeid of the sender (the serialnumber of the RTCU that sent the message)</param>
        /// <param name="data">The data sent by the RTCU. Max size is 480 bytes.</param>
        /// <param name="length">The length of data sent by the RTCU</param>
        /// <param name="arg">A user supplied 32 bit variable that was set when vsmsgwInit() was called</param>
        /// <returns>This is the return code that should be returned to the RTCU Unit, 0 if OK, else error</returns>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate UInt16 cbfuncPackage(
            Int32 HisNodeID,
            [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] Byte[] data,
            Int32 length,
            IntPtr arg
        );

        #endregion

        #region "Decleration of function in dll files"

        /// <summary>
        /// This function will return this nodes nodeid.
        /// 
        /// This function is used especially when a dynamic node-id is requested (by setting MyNodeID to 0 in the vsmsgwInit() function
        /// </summary>
        /// <param name="MyNodeID">Will be updated with current node id</param>
        /// <returns>Returns 0 if OK, else error (Look at GWRC_xx codes)</returns>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#1")]
        static public extern GWRC GetMyNodeID(ref UInt32 MyNodeID);

        /// <summary>
        /// Initialize the connection to the RTCU Gateway
        /// </summary>
        /// <param name="MyNodeID">The nodeid for the PC application. If set to 0, it will be assigned by the RTCU Gateway</param>
        /// <param name="GWIP">The IP address (or symbolic name) of the RTCU Gateway</param>
        /// <param name="GWPort">The pportnumber the RTCU Gateway listens on</param>
        /// <param name="GWKey">The key value (an 8 character password) used to access the RTCU Gateway. Must be null terminated,</param>
        /// <param name="SMSText">A callbackfunction that will be called whenever an Text SMS is total_received</param>
        /// <param name="SMSPDU">A callbackfunction that will be called whenever a PDU SMS is total_received</param>
        /// <param name="arg">A user supplied 32 bit variable that will be supplied to the callback functions when they get called</param>
        /// <returns>Returns 0 if OK, else error (Look at GWRC_xx codes)</returns>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#2")
        ]
        static public extern GWRC Init(
            UInt32 MyNodeID,
            String GWIP,
            Int32 GWPort,
            String GWKey,
            cbfuncText SMSText,
            cbfuncPDU SMSPDU,
            IntPtr arg);

        /// <summary>
        /// Initialize the connection to the RTCU Gateway
        /// </summary>
        /// <param name="MyNodeID">The nodeid for the PC application. If set to 0, it will be assigned by the RTCU Gateway</param>
        /// <param name="GWIP">The IP address (or symbolic name) of the RTCU Gateway</param>
        /// <param name="GWPort">The pportnumber the RTCU Gateway listens on</param>
        /// <param name="GWKey">The key value (an 8 character password) used to access the RTCU Gateway. Must be null terminated,</param>
        /// <param name="CryptKey">The encryption key used to access the RTCU Gateway (must be 16 bytes).</param>
        /// <returns>Returns 0 if OK, else error (Look at GWRC_xx codes)</returns>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#3")
        ]
        static public extern GWRC InitAdv(
            UInt32 MyNodeID,
            String GWIP,
            Int32 GWPort,
            String GWKey,
            [MarshalAs(UnmanagedType.LPArray, SizeConst = 16)] Byte[] CryptKey);

        /// <summary>
        /// Checks if connection to RTCU Gateway has bee etablished
        /// </summary>
        /// <returns>Returns true if connected to the RTCU Gateway</returns>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#4")
        ]
        static public extern Boolean IsConnected();

        /// <summary>
        /// Send a data package to the specified NodeID, the returncode from the RTCU unit, will be contained in rc
        /// </summary>
        /// <param name="HisNodeID">The nodenumber (serialnumber) of the receiving RTCU unit</param>
        /// <param name="data">The data to send to the receiving RTCU unit. Maximum size is 480 bytes.</param>
        /// <param name="length">The length of data to send</param>
        /// <param name="rc">The return code from the receiving RTCU unit, 0 if OK, else error</param>
        /// <returns>Returns 0 if OK, else error (Look at GWRC_xx codes)</returns>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#5")
        ]
        static public extern GWRC SendDataPackage(
            UInt32 HisNodeID,
            [MarshalAs(UnmanagedType.LPArray)] Byte[] data,
            Int32 length,
            ref Int32 rc);

        /// <summary>
        /// Send a PDU SMS message to the specified NodeID, the returncode from the RTCU unit, will be contained in rc
        /// </summary>
        /// <param name="HisNodeID">The nodenumber (serialnumber) of the receiving RTCU unit</param>
        /// <param name="data">The data to send to the receiving RTCU unitMaximum size is 140 bytes.</param>
        /// <param name="length">The length of data to send</param>
        /// <param name="rc">The return code from the receiving RTCU unit, 0 if OK, else error</param>
        /// <returns>Returns 0 if OK, else error (Look at GWRC_xx codes)</returns>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#6")
        ]
        static public extern GWRC SendPDU(
            UInt32 HisNodeID,
            [MarshalAs(UnmanagedType.LPArray)] Byte[] data,
            Int32 length,
            ref Int32 rc);

        /// <summary>
        /// Send a Text SMS message to the specified NodeID, the returncode from the RTCU unit, will be contained in rc.
        /// </summary>
        /// <param name="HisNodeID">The nodenumber (serialnumber) of the receiving RTCU unit</param>
        /// <param name="str">The string to send to the receiving RTCU unit. Maximum size is 160 characters + NULL terminator.</param>
        /// <param name="rc">The return code from the receiving RTCU unit, 0 if OK, else error</param>
        /// <returns>Returns 0 if OK, else error (Look at GWRC_xx codes)</returns>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#7")
        ]
        static public extern GWRC SendSMS(
            UInt32 HisNodeID,
            [MarshalAs(UnmanagedType.LPStr)] String str,
            ref Int32 rc);

        /// <summary>
        /// Set callback function that will be called whenever a data package is total_received
        /// </summary>
        /// <param name="PACKAGE">A callbackfunction that will be called whenever a data package is total_received.</param>
        /// <param name="arg">A user supplied 32 bit variable that will be supplied to the callback function when it gets called.</param>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#8")
        ]
        static public extern void SetDataPackageCB(cbfuncPackage PACKAGE, IntPtr arg);

        /// <summary>
        /// Set callback function that will be called whenever a PDU SMS is total_received
        /// </summary>
        /// <param name="SMSPDU">A callbackfunction that will be called whenever a PDU SMS is total_received.</param>
        /// <param name="arg">A user supplied 32 bit variable that will be supplied to the callback function when it gets called.</param>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#9")
        ]
        static public extern void SetSMSPDUCB(cbfuncPDU SMSPDU, IntPtr arg);

        /// <summary>
        /// Set callback function that will be called whenever an Text SMS is total_received
        /// </summary>
        /// <param name="SMSText">A callbackfunction that will be called whenever an Text SMS is total_received.</param>
        /// <param name="arg">A user supplied 32 bit variable that will be supplied to the callback function when it gets called.</param>
        [DllImport(
            @".\LogicIO RTCU library\VSMSGW",
            CallingConvention = CallingConvention.Cdecl,
            CharSet = CharSet.Ansi,
            EntryPoint = "#10")
        ]
        static public extern void SetSMSTextCB(cbfuncText SMSText, IntPtr arg);

        #endregion
    }
}
